@@ -1,93 +1,25 @@
 from __future__ import print_function
 
 import six
-
 import json
-import inspect
-import base64
-import copy
-import re
-from collections import OrderedDict
 from functools import partial
 
 import googleapiclient.errors
-import googleapiclient.discovery
 import numpy as np
 
-from future.utils import with_metaclass
-
 from cloudvolume.threaded_queue import ThreadedQueue
-from .secrets import google_credentials, PROJECT_NAME, QUEUE_NAME, QUEUE_TYPE
-from .appengine_queue_api import AppEngineTaskQueue
 
-__all__ = [ 'RegisteredTask', 'TaskQueue' ]
+from .appengine_queue_api import AppEngineTaskQueueAPI
+from .google_queue_api import GoogleTaskQueueAPI
+from .aws_queue_api import AWSTaskQueueAPI
+from .registered_task import RegisteredTask, payloadBase64Decode
+from .secrets import PROJECT_NAME, QUEUE_NAME, QUEUE_TYPE
 
-registry = {}
-
-def register_class(target_class):
-    registry[target_class.__name__] = target_class
-
-def deserialize(data):
-    params = json.loads(data)
-    name = params['class']
-    target_class = registry[name]
-    del params['class']
-    return target_class(**params)
-
-def payloadBase64Decode(payload):
-    decoded_string = base64.b64decode(payload).encode('ascii')
-    return deserialize(decoded_string)
-
-class Meta(type):
-    def __new__(meta, name, bases, class_dict):
-        cls = type.__new__(meta, name, bases, class_dict)
-        register_class(cls)
-        cls._arg_names = inspect.getargspec(class_dict['__init__'])[0][1:]
-        return cls
-
-class RegisteredTask(with_metaclass(Meta)):
-    def __init__(self, *arg_values):
-        self._args = OrderedDict(zip(self._arg_names, arg_values))
-
-    @classmethod
-    def deserialize(cls, base64data):
-        obj = deserialize(base64data)
-        assert type(obj) == cls
-        return obj
-        
-    def serialize(self):
-        d = copy.deepcopy(self._args)
-        d['class'] = self.__class__.__name__
-
-        def serializenumpy(obj):
-            if isinstance(obj, np.ndarray):
-                return obj.tolist()
-            return obj
-
-        return json.dumps(d, default=serializenumpy)
-
-    @property
-    def payloadBase64(self):
-        return base64.b64encode(self.serialize().encode('utf8'))
-
-    @property
-    def id(self):
-        return self._id
-
-    def __repr__(self):
-        
-        string = self.__class__.__name__ + "("
-        for arg_name, arg_value in six.iteritems(self._args):
-            if type(arg_value) is str or type(arg_value) is unicode:
-                string += "{}='{}',".format(arg_name, arg_value)
-            else:
-                string += "{}={},".format(arg_name, arg_value)
-
-        # remove the last comma if necessary
-        if string[-1] == ',':
-            string = string[:-1]
-
-        return string + ")"  
+def totask(task):
+    print(task)
+    taskobj = payloadBase64Decode(task['payloadBase64'])
+    taskobj._id = task['id']
+    return taskobj
 
 class TaskQueue(ThreadedQueue):
     """
@@ -107,24 +39,29 @@ class TaskQueue(ThreadedQueue):
         def __init__(self):
             super(LookupError, self).__init__('Queue Empty')
 
-    def __init__(self, n_threads=40, project=PROJECT_NAME,
-                 queue_name=QUEUE_NAME, queue_server=QUEUE_TYPE):
+    def __init__(self, n_threads=40, project=PROJECT_NAME, region=None,
+                 queue_name=QUEUE_NAME, queue_server=QUEUE_TYPE, qurl=None):
 
-        self._project = 's~' + project # s~ means North America, e~ means Europe
+        self._project = project
+        self._region = region
         self._queue_name = queue_name
         self._queue_server = queue_server
+        self._qurl = qurl
         self._api = self._initialize_interface()
 
         super(TaskQueue, self).__init__(n_threads) # creates self._queue
 
     # This is key to making sure threading works. Don't refactor this method away.
     def _initialize_interface(self):
-        if self._queue_server == 'appengine':
-            return AppEngineTaskQueue()
-        elif self._queue_server == 'pull-queue':
-            return googleapiclient.discovery.build('taskqueue', 'v1beta2', credentials=google_credentials)
+        server = self._queue_server.lower()
+        if server == 'appengine':
+            return AppEngineTaskQueueAPI(project=self._project, queue_name=self._queue_name)
+        elif server in ('pull-queue', 'google'):
+            return GoogleTaskQueueAPI(project=self._project, queue_name=self._queue_name)
+        elif server in ('sqs', 'aws'):
+            return AWSTaskQueueAPI(qurl=self._qurl)
         else:
-            raise ValueError('Unkown server ' + self._queue_server)
+            raise NotImplementedError('Unknown server ' + self._queue_server)
 
     @property
     def enqueued(self):
@@ -139,8 +76,7 @@ class TaskQueue(ThreadedQueue):
         
         Returns: (int) number of tasks in cloud queue
         """
-        tqinfo = self.get()
-        return tqinfo['stats']['totalTasks']
+        return self._api.enqueued
         
     def _consume_queue_execution(self, fn):
         try:
@@ -169,11 +105,7 @@ class TaskQueue(ThreadedQueue):
         }
 
         def cloud_insertion(api):
-            api.tasks().insert(
-                project=self._project,
-                taskqueue=self._queue_name,
-                body=body,
-            ).execute(num_retries=6)
+            api.insert(body)
 
         if len(self._threads):
             self.put(cloud_insertion)
@@ -182,15 +114,11 @@ class TaskQueue(ThreadedQueue):
 
         return self
 
-    def get(self):
+    def status(self):
         """
         Gets information about the TaskQueue
         """
-        return self._api.taskqueues().get(
-            project=self._project,
-            taskqueue=self._queue_name,
-            getStats=True,
-        ).execute(num_retries=6)
+        return self._api.get(getStats=True)
 
     def get_task(self, tid):
         """
@@ -198,52 +126,40 @@ class TaskQueue(ThreadedQueue):
         tid is a unique string Google provides 
         e.g. '7c6e81c9b7ab23f0'
         """
-        return self._api.tasks().get(
-            project=self._project,
-            taskqueue=self._queue_name,
-            task=tid,
-        ).execute(num_retries=6)
+        return self._api.get(tid)
 
     def list(self):
         """
         Lists all non-deleted Tasks in a TaskQueue, 
         whether or not they are currently leased, up to a maximum of 100.
         """
-        return self._api.tasks().list(
-            project=self._project, 
-            taskqueue=self._queue_name
-        ).execute(num_retries=6)
+        return [ totask(x) for x in self._api.list() ]
 
-    def update(self, task):
-        """
-        Update the duration of a task lease.
-        Required query parameters: newLeaseSeconds
-        """
-        raise NotImplemented
+    def renew_lease(self, task, seconds):
+        """Update the duration of a task lease."""
+        return self._api.renew_lease(task, seconds)
 
-    def lease(self, tag=None):
+    def cancel_lease(self, task):
+        return self._api.cancel_lease(task)
+
+    def lease(self, seconds=600, num_tasks=1, tag=None):
         """
         Acquires a lease on the topmost N unowned tasks in the specified queue.
         Required query parameters: leaseSecs, numTasks
         """
         tag = tag if tag else None
-        
-        tasks = self._api.tasks().lease(
-            project=self._project,
-            taskqueue=self._queue_name, 
-            numTasks=1, 
-            leaseSecs=600,
+        tasks = self._api.lease(
+            numTasks=num_tasks, 
+            seconds=seconds,
             groupByTag=(tag is not None),
             tag=tag,
-        ).execute(num_retries=6)
+        )
 
-        if not 'items' in tasks:
+        if not len(tasks):
             raise TaskQueue.QueueEmpty
-          
-        task_json = tasks['items'][0]
-        t = payloadBase64Decode(task_json['payloadBase64'])
-        t._id =  task_json['id']
-        return t
+
+        task = tasks[0]
+        return totask(task)
 
     def patch(self):
         """
@@ -254,14 +170,31 @@ class TaskQueue(ThreadedQueue):
 
     def purge(self):
         """Deletes all tasks in the queue."""
-        while True:
-            lst = self.list()
-            if 'items' not in lst:
-                break
+        try:
+            return self._api.purge()
+        except AttributeError:
+            while True:
+                lst = self.list()
+                if len(lst) == 0:
+                    break
+
+                for task in lst:
+                    self.delete(task)
+                self.wait()
+            return self
+
+    def acknowledge(self, task_id):
+        if isinstance(task_id, RegisteredTask):
+            task_id = task_id.id
+
+        def cloud_delete(api):
+            api.acknowledge(task_id)
+
+        if len(self._threads):
+            self.put(cloud_delete)
+        else:
+            cloud_delete(self._api)
 
-            for task in lst['items']:
-                self.delete(task['id'])
-            self.wait()
         return self
 
     def delete(self, task_id):
@@ -270,19 +203,16 @@ class TaskQueue(ThreadedQueue):
             task_id = task_id.id
 
         def cloud_delete(api):
-            api.tasks().delete(
-                project=self._project,
-                taskqueue=self._queue_name,
-                task=task_id,
-            ).execute(num_retries=6)
+            api.delete(task_id)
 
         if len(self._threads):
             self.put(cloud_delete)
         else:
             cloud_delete(self._api)
+
         return self
 
-class MockTaskQueue():
+class MockTaskQueue(object):
     def __init__(self, queue_name='', queue_server=''):
         pass
 
